@@ -1,33 +1,37 @@
-from __future__ import unicode_literals
-
-from django.contrib.auth import authenticate
+from django import forms
+from django.contrib.auth import authenticate, get_user_model
 from django.contrib.auth.tokens import default_token_generator
 from django.db.models import Q
 from django.db.models.manager import Manager
-from django import forms
 from django.utils.http import int_to_base36
-from django.utils.translation import ugettext, ugettext_lazy as _
+from django.utils.translation import gettext
+from django.utils.translation import gettext_lazy as _
 
-from mezzanine.accounts import (get_profile_model, get_profile_user_fieldname,
-                                get_profile_for_user, ProfileNotConfigured)
+from mezzanine.accounts import (
+    ProfileNotConfigured,
+    get_profile_for_user,
+    get_profile_model,
+    get_profile_user_fieldname,
+)
 from mezzanine.conf import settings
 from mezzanine.core.forms import Html5Mixin
-from mezzanine.utils.models import get_user_model
 from mezzanine.utils.urls import slugify, unique_slug
 
-
 User = get_user_model()
 
-_exclude_fields = tuple(settings.ACCOUNTS_PROFILE_FORM_EXCLUDE_FIELDS)
+_exclude_fields = tuple(getattr(settings, "ACCOUNTS_PROFILE_FORM_EXCLUDE_FIELDS", ()))
 
-# If a profile model has been configured with the ``AUTH_PROFILE_MODULE``
+# If a profile model has been configured with the ``ACCOUNTS_PROFILE_MODEL``
 # setting, create a model form for it that will have its fields added to
 # ``ProfileForm``.
 try:
+
     class ProfileFieldsForm(forms.ModelForm):
         class Meta:
             model = get_profile_model()
             exclude = (get_profile_user_fieldname(),) + _exclude_fields
+
+
 except ProfileNotConfigured:
     pass
 
@@ -43,9 +47,11 @@ class LoginForm(Html5Mixin, forms.Form):
     """
     Fields for login.
     """
+
     username = forms.CharField(label=username_label)
-    password = forms.CharField(label=_("Password"),
-                               widget=forms.PasswordInput(render_value=False))
+    password = forms.CharField(
+        label=_("Password"), widget=forms.PasswordInput(render_value=False)
+    )
 
     def clean(self):
         """
@@ -56,10 +62,9 @@ class LoginForm(Html5Mixin, forms.Form):
         password = self.cleaned_data.get("password")
         self._user = authenticate(username=username, password=password)
         if self._user is None:
-            raise forms.ValidationError(
-                             ugettext("Invalid username/email and password"))
+            raise forms.ValidationError(gettext("Invalid username/email and password"))
         elif not self._user.is_active:
-            raise forms.ValidationError(ugettext("Your account is inactive"))
+            raise forms.ValidationError(gettext("Your account is inactive"))
         return self.cleaned_data
 
     def save(self):
@@ -72,14 +77,16 @@ class LoginForm(Html5Mixin, forms.Form):
 class ProfileForm(Html5Mixin, forms.ModelForm):
     """
     ModelForm for auth.User - used for signup and profile update.
-    If a Profile model is defined via ``AUTH_PROFILE_MODULE``, its
+    If a Profile model is defined via ``ACCOUNTS_PROFILE_MODEL``, its
     fields are injected into the form.
     """
 
-    password1 = forms.CharField(label=_("Password"),
-                                widget=forms.PasswordInput(render_value=False))
-    password2 = forms.CharField(label=_("Password (again)"),
-                                widget=forms.PasswordInput(render_value=False))
+    password1 = forms.CharField(
+        label=_("Password"), widget=forms.PasswordInput(render_value=False)
+    )
+    password2 = forms.CharField(
+        label=_("Password (again)"), widget=forms.PasswordInput(render_value=False)
+    )
 
     class Meta:
         model = User
@@ -87,12 +94,13 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
         exclude = _exclude_fields
 
     def __init__(self, *args, **kwargs):
-        super(ProfileForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._signup = self.instance.id is None
-        user_fields = User._meta.get_all_field_names()
+        user_fields = {f.name for f in User._meta.get_fields()}
         try:
-            self.fields["username"].help_text = ugettext(
-                        "Only letters, numbers, dashes or underscores please")
+            self.fields["username"].help_text = gettext(
+                "Only letters, numbers, dashes or underscores please"
+            )
         except KeyError:
             pass
         for field in self.fields:
@@ -107,8 +115,9 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
                 if not self._signup:
                     self.fields[field].required = False
                     if field == "password1":
-                        self.fields[field].help_text = ugettext(
-                        "Leave blank unless you want to change your password")
+                        self.fields[field].help_text = gettext(
+                            "Leave blank unless you want " "to change your password"
+                        )
 
         # Add any profile fields to the form.
         try:
@@ -135,15 +144,17 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
         username = self.cleaned_data.get("username")
         if username.lower() != slugify(username).lower():
             raise forms.ValidationError(
-                ugettext("Username can only contain letters, numbers, dashes "
-                         "or underscores."))
+                gettext(
+                    "Username can only contain letters, numbers, dashes "
+                    "or underscores."
+                )
+            )
         lookup = {"username__iexact": username}
         try:
             User.objects.exclude(id=self.instance.id).get(**lookup)
         except User.DoesNotExist:
             return username
-        raise forms.ValidationError(
-                            ugettext("This username is already registered"))
+        raise forms.ValidationError(gettext("This username is already registered"))
 
     def clean_password2(self):
         """
@@ -156,11 +167,12 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
         if password1:
             errors = []
             if password1 != password2:
-                errors.append(ugettext("Passwords do not match"))
+                errors.append(gettext("Passwords do not match"))
             if len(password1) < settings.ACCOUNTS_MIN_PASSWORD_LENGTH:
                 errors.append(
-                        ugettext("Password must be at least %s characters") %
-                        settings.ACCOUNTS_MIN_PASSWORD_LENGTH)
+                    gettext("Password must be at least %s characters")
+                    % settings.ACCOUNTS_MIN_PASSWORD_LENGTH
+                )
             if errors:
                 self._errors["password1"] = self.error_class(errors)
         return password2
@@ -173,8 +185,7 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
         qs = User.objects.exclude(id=self.instance.id).filter(email=email)
         if len(qs) == 0:
             return email
-        raise forms.ValidationError(
-                                ugettext("This email is already registered"))
+        raise forms.ValidationError(gettext("This email is already registered"))
 
     def save(self, *args, **kwargs):
         """
@@ -186,13 +197,18 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
         """
 
         kwargs["commit"] = False
-        user = super(ProfileForm, self).save(*args, **kwargs)
+        user = super().save(*args, **kwargs)
         try:
             self.cleaned_data["username"]
         except KeyError:
             if not self.instance.username:
-                username = "%(first_name)s %(last_name)s" % self.cleaned_data
-                if not username.strip():
+                try:
+                    username = (
+                        "%(first_name)s %(last_name)s" % self.cleaned_data
+                    ).strip()
+                except KeyError:
+                    username = ""
+                if not username:
                     username = self.cleaned_data["email"].split("@")[0]
                 qs = User.objects.exclude(id=self.instance.id)
                 user.username = unique_slug(qs, "username", slugify(username))
@@ -216,16 +232,17 @@ class ProfileForm(Html5Mixin, forms.ModelForm):
             pass
 
         if self._signup:
-            settings.use_editable()
-            if (settings.ACCOUNTS_VERIFICATION_REQUIRED or
-                settings.ACCOUNTS_APPROVAL_REQUIRED):
+            if (
+                settings.ACCOUNTS_VERIFICATION_REQUIRED
+                or settings.ACCOUNTS_APPROVAL_REQUIRED
+            ):
                 user.is_active = False
                 user.save()
             else:
                 token = default_token_generator.make_token(user)
-                user = authenticate(uidb36=int_to_base36(user.id),
-                                    token=token,
-                                    is_active=True)
+                user = authenticate(
+                    uidb36=int_to_base36(user.id), token=token, is_active=True
+                )
         return user
 
     def get_profile_fields_form(self):
@@ -249,8 +266,7 @@ class PasswordResetForm(Html5Mixin, forms.Form):
         try:
             user = User.objects.get(username_or_email, is_active=True)
         except User.DoesNotExist:
-            raise forms.ValidationError(
-                             ugettext("Invalid username/email"))
+            raise forms.ValidationError(gettext("Invalid username/email"))
         else:
             self._user = user
         return self.cleaned_data
