@@ -1,23 +1,51 @@
-from __future__ import unicode_literals
-from future.builtins import int, zip
-
 from functools import reduce
-from operator import ior, iand
+from operator import iand, ior
 from string import punctuation
 
+import django
+from django.apps import apps
+from django.contrib.sites.managers import CurrentSiteManager as DjangoCSM
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Manager, Q, CharField, TextField, get_models
+from django.db.models import CharField, Manager, Q, TextField
 from django.db.models.manager import ManagerDescriptor
 from django.db.models.query import QuerySet
-from django.contrib.sites.managers import CurrentSiteManager as DjangoCSM
 from django.utils.timezone import now
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from mezzanine.conf import settings
-from mezzanine.utils.models import get_model
 from mezzanine.utils.sites import current_site_id
 from mezzanine.utils.urls import home_slug
 
+if django.VERSION >= (1, 10):
+
+    class ManagerDescriptor(ManagerDescriptor):
+        """
+        This class exists purely to skip the abstract model check
+        in the __get__ method of Django's ManagerDescriptor.
+        """
+
+        def __get__(self, instance, cls=None):
+            if instance is not None:
+                raise AttributeError(
+                    "Manager isn't accessible via %s instances" % cls.__name__
+                )
+
+            # In ManagerDescriptor.__get__, an exception is raised here
+            # if cls is abstract
+
+            if cls._meta.swapped:
+                raise AttributeError(
+                    "Manager isn't available; "
+                    "'%s.%s' has been swapped for '%s'"
+                    % (
+                        cls._meta.app_label,
+                        cls._meta.object_name,
+                        cls._meta.swapped,
+                    )
+                )
+
+            return cls._meta.managers_map[self.manager.name]
+
 
 class PublishedManager(Manager):
     """
@@ -32,12 +60,14 @@ class PublishedManager(Manager):
         current date when specified.
         """
         from mezzanine.core.models import CONTENT_STATUS_PUBLISHED
+
         if for_user is not None and for_user.is_staff:
             return self.all()
         return self.filter(
             Q(publish_date__lte=now()) | Q(publish_date__isnull=True),
             Q(expiry_date__gte=now()) | Q(expiry_date__isnull=True),
-            Q(status=CONTENT_STATUS_PUBLISHED))
+            Q(status=CONTENT_STATUS_PUBLISHED),
+        )
 
     def get_by_natural_key(self, slug):
         return self.get(slug=slug)
@@ -70,7 +100,7 @@ class SearchableQuerySet(QuerySet):
         self._search_ordered = False
         self._search_terms = set()
         self._search_fields = kwargs.pop("search_fields", {})
-        super(SearchableQuerySet, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def search(self, query, search_fields=None):
         """
@@ -80,7 +110,7 @@ class SearchableQuerySet(QuerySet):
         to require and exclude.
         """
 
-        #### DETERMINE FIELDS TO SEARCH ###
+        # ### DETERMINE FIELDS TO SEARCH ###
 
         # Use search_fields arg if given, otherwise use search_fields
         # initially configured by the manager class.
@@ -89,26 +119,31 @@ class SearchableQuerySet(QuerySet):
         if not self._search_fields:
             return self.none()
 
-        #### BUILD LIST OF TERMS TO SEARCH FOR ###
+        # ### BUILD LIST OF TERMS TO SEARCH FOR ###
 
         # Remove extra spaces, put modifiers inside quoted terms.
-        terms = " ".join(query.split()).replace("+ ", "+")     \
-                                       .replace('+"', '"+')    \
-                                       .replace("- ", "-")     \
-                                       .replace('-"', '"-')    \
-                                       .split('"')
+        terms = (
+            " ".join(query.split())
+            .replace("+ ", "+")
+            .replace('+"', '"+')
+            .replace("- ", "-")
+            .replace('-"', '"-')
+            .split('"')
+        )
         # Strip punctuation other than modifiers from terms and create
         # terms list, first from quoted terms and then remaining words.
-        terms = [("" if t[0:1] not in "+-" else t[0:1]) + t.strip(punctuation)
-            for t in terms[1::2] + "".join(terms[::2]).split()]
+        terms = [
+            ("" if t[0:1] not in "+-" else t[0:1]) + t.strip(punctuation)
+            for t in terms[1::2] + "".join(terms[::2]).split()
+        ]
         # Remove stop words from terms that aren't quoted or use
         # modifiers, since words with these are an explicit part of
         # the search query. If doing so ends up with an empty term
         # list, then keep the stop words.
-        terms_no_stopwords = [t for t in terms if t.lower() not in
-            settings.STOP_WORDS]
-        get_positive_terms = lambda terms: [t.lower().strip(punctuation)
-            for t in terms if t[0:1] != "-"]
+        terms_no_stopwords = [t for t in terms if t.lower() not in settings.STOP_WORDS]
+        get_positive_terms = lambda terms: [
+            t.lower().strip(punctuation) for t in terms if t[0:1] != "-"
+        ]
         positive_terms = get_positive_terms(terms_no_stopwords)
         if positive_terms:
             terms = terms_no_stopwords
@@ -121,15 +156,35 @@ class SearchableQuerySet(QuerySet):
         else:
             self._search_terms.update(positive_terms)
 
-        #### BUILD QUERYSET FILTER ###
+        # ### BUILD QUERYSET FILTER ###
 
         # Create the queryset combining each set of terms.
-        excluded = [reduce(iand, [~Q(**{"%s__icontains" % f: t[1:]}) for f in
-            self._search_fields.keys()]) for t in terms if t[0:1] == "-"]
-        required = [reduce(ior, [Q(**{"%s__icontains" % f: t[1:]}) for f in
-            self._search_fields.keys()]) for t in terms if t[0:1] == "+"]
-        optional = [reduce(ior, [Q(**{"%s__icontains" % f: t}) for f in
-            self._search_fields.keys()]) for t in terms if t[0:1] not in "+-"]
+        excluded = [
+            reduce(
+                iand,
+                [
+                    ~Q(**{"%s__icontains" % f: t[1:]})
+                    for f in self._search_fields.keys()
+                ],
+            )
+            for t in terms
+            if t[0:1] == "-"
+        ]
+        required = [
+            reduce(
+                ior,
+                [Q(**{"%s__icontains" % f: t[1:]}) for f in self._search_fields.keys()],
+            )
+            for t in terms
+            if t[0:1] == "+"
+        ]
+        optional = [
+            reduce(
+                ior, [Q(**{"%s__icontains" % f: t}) for f in self._search_fields.keys()]
+            )
+            for t in terms
+            if t[0:1] not in "+-"
+        ]
         queryset = self
         if excluded:
             queryset = queryset.filter(reduce(iand, excluded))
@@ -145,9 +200,11 @@ class SearchableQuerySet(QuerySet):
         """
         Ensure attributes are copied to subsequent queries.
         """
-        for attr in ("_search_terms", "_search_fields", "_search_ordered"):
-            kwargs[attr] = getattr(self, attr)
-        return super(SearchableQuerySet, self)._clone(*args, **kwargs)
+        clone = super()._clone(*args, **kwargs)
+        clone._search_terms = self._search_terms
+        clone._search_fields = self._search_fields
+        clone._search_ordered = self._search_ordered
+        return clone
 
     def order_by(self, *field_names):
         """
@@ -155,9 +212,9 @@ class SearchableQuerySet(QuerySet):
         """
         if not self._search_ordered:
             self._search_ordered = len(self._search_terms) > 0
-        return super(SearchableQuerySet, self).order_by(*field_names)
+        return super().order_by(*field_names)
 
-    def iterator(self):
+    def annotate_scores(self):
         """
         If search has occurred and no ordering has occurred, decorate
         each result with the number of search terms so that it can be
@@ -171,7 +228,7 @@ class SearchableQuerySet(QuerySet):
         we assume one match for one of the fields, and use the average
         weight of all search fields with relationships.
         """
-        results = super(SearchableQuerySet, self).iterator()
+        results = super().iterator()
         if self._search_terms and not self._search_ordered:
             results = list(results)
             for i, result in enumerate(results):
@@ -186,6 +243,12 @@ class SearchableQuerySet(QuerySet):
                             count += field_value.lower().count(term) * weight
                 if not count and related_weights:
                     count = int(sum(related_weights) / len(related_weights))
+
+                if result.publish_date:
+                    age = (now() - result.publish_date).total_seconds()
+                    if age > 0:
+                        count = count / age ** settings.SEARCH_AGE_SCALE_FACTOR
+
                 results[i].result_count = count
             return iter(results)
         return results
@@ -201,12 +264,12 @@ class SearchableManager(Manager):
 
     def __init__(self, *args, **kwargs):
         self._search_fields = kwargs.pop("search_fields", {})
-        super(SearchableManager, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def get_search_fields(self):
         """
         Returns the search field names mapped to weights as a dict.
-        Used in ``get_query_set`` below to tell ``SearchableQuerySet``
+        Used in ``get_queryset`` below to tell ``SearchableQuerySet``
         which search fields to use. Also used by ``DisplayableAdmin``
         to populate Django admin's ``search_fields`` attribute.
 
@@ -232,23 +295,23 @@ class SearchableManager(Manager):
                 search_fields.update(search_fields_to_dict(super_fields))
         if not search_fields:
             search_fields = []
-            for f in self.model._meta.fields:
+            for f in self.model._meta.get_fields():
                 if isinstance(f, (CharField, TextField)):
                     search_fields.append(f.name)
             search_fields = search_fields_to_dict(search_fields)
         return search_fields
 
-    def get_query_set(self):
+    def get_queryset(self):
         search_fields = self.get_search_fields()
         return SearchableQuerySet(self.model, search_fields=search_fields)
 
     def contribute_to_class(self, model, name):
         """
-        Django 1.5 explicitly prevents managers being accessed from
-        abstract classes, which is behaviour the search API has relied
-        on for years. Here we reinstate it.
+        Newer versions of Django explicitly prevent managers being
+        accessed from abstract classes, which is behaviour the search
+        API has always relied on. Here we reinstate it.
         """
-        super(SearchableManager, self).contribute_to_class(model, name)
+        super().contribute_to_class(model, name)
         setattr(model, name, ManagerDescriptor(self))
 
     def search(self, *args, **kwargs):
@@ -260,8 +323,8 @@ class SearchableManager(Manager):
         if not settings.SEARCH_MODEL_CHOICES:
             # No choices defined - build a list of leaf models (those
             # without subclasses) that inherit from Displayable.
-            models = [m for m in get_models() if issubclass(m, self.model)]
-            parents = reduce(ior, [m._meta.get_parent_list() for m in models])
+            models = [m for m in apps.get_models() if issubclass(m, self.model)]
+            parents = reduce(ior, [set(m._meta.get_parent_list()) for m in models])
             models = [m for m in models if m not in parents]
         elif getattr(self.model._meta, "abstract", False):
             # When we're combining model subclasses for an abstract
@@ -278,17 +341,19 @@ class SearchableManager(Manager):
             errors = []
             for name in settings.SEARCH_MODEL_CHOICES:
                 try:
-                    model = get_model(*name.split(".", 1))
+                    model = apps.get_model(*name.split(".", 1))
                 except LookupError:
                     errors.append(name)
                 else:
                     search_choices.add(model)
             if errors:
-                raise ImproperlyConfigured("Could not load the model(s) "
-                        "%s defined in the 'SEARCH_MODEL_CHOICES' setting."
-                        % ", ".join(errors))
+                raise ImproperlyConfigured(
+                    "Could not load the model(s) "
+                    "%s defined in the 'SEARCH_MODEL_CHOICES' setting."
+                    % ", ".join(errors)
+                )
 
-            for model in get_models():
+            for model in apps.get_models():
                 # Model is actually a subclasses of what we're
                 # searching (eg Displayabale)
                 is_subclass = issubclass(model, self.model)
@@ -318,8 +383,8 @@ class SearchableManager(Manager):
             try:
                 queryset = model.objects.published(for_user=user)
             except AttributeError:
-                queryset = model.objects.get_query_set()
-            all_results.extend(queryset.search(*args, **kwargs))
+                queryset = model.objects.get_queryset()
+            all_results.extend(queryset.search(*args, **kwargs).annotate_scores())
         return sorted(all_results, key=lambda r: r.result_count, reverse=True)
 
 
@@ -333,25 +398,21 @@ class CurrentSiteManager(DjangoCSM):
     to ``settings.SITE_ID`` if none of those match a site.
     """
 
+    use_in_migrations = False
+
     def __init__(self, field_name=None, *args, **kwargs):
         super(DjangoCSM, self).__init__(*args, **kwargs)
         self.__field_name = field_name
         self.__is_validated = False
 
-    def get_query_set(self):
+    def get_queryset(self):
         if not self.__is_validated:
-            try:
-                # Django <= 1.6
-                self._validate_field_name()
-            except AttributeError:
-                # Django >= 1.7: will populate "self.__field_name".
-                self._get_field_name()
+            self._get_field_name()
         lookup = {self.__field_name + "__id__exact": current_site_id()}
-        return super(DjangoCSM, self).get_query_set().filter(**lookup)
+        return super(DjangoCSM, self).get_queryset().filter(**lookup)
 
 
-class DisplayableManager(CurrentSiteManager, PublishedManager,
-                         SearchableManager):
+class DisplayableManager(CurrentSiteManager, PublishedManager, SearchableManager):
     """
     Manually combines ``CurrentSiteManager``, ``PublishedManager``
     and ``SearchableManager`` for the ``Displayable`` model.
@@ -364,14 +425,21 @@ class DisplayableManager(CurrentSiteManager, PublishedManager,
         instances, including a fake homepage instance if none exists.
         Used in ``mezzanine.core.sitemaps``.
         """
-        home = self.model(title=_("Home"))
+
+        class Home:
+            title = _("Home")
+
+        home = Home()
         setattr(home, "get_absolute_url", home_slug)
         items = {home.get_absolute_url(): home}
-        for model in get_models():
+        for model in apps.get_models():
             if issubclass(model, self.model):
-                for item in (model.objects.published(for_user=for_user)
-                                  .filter(**kwargs)
-                                  .exclude(slug__startswith="http://")
-                                  .exclude(slug__startswith="https://")):
-                    items[item.get_absolute_url()] = item
+                if hasattr(model.objects, "published"):
+                    for item in (
+                        model.objects.published(for_user=for_user)
+                        .filter(**kwargs)
+                        .exclude(slug__startswith="http://")
+                        .exclude(slug__startswith="https://")
+                    ):
+                        items[item.get_absolute_url()] = item
         return items
